@@ -1,4 +1,22 @@
 # -*- coding: utf-8 -*-
+
+# Copyright © 2012-2017 marmuta <marmvta@gmail.com>
+#
+# This file is part of Onboard.
+#
+# Onboard is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 3 of the License, or
+# (at your option) any later version.
+#
+# Onboard is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program. If not, see <http://www.gnu.org/licenses/>.
+
 """ Window manipulation and other helpers """
 
 from __future__ import division, print_function, unicode_literals
@@ -6,21 +24,21 @@ from __future__ import division, print_function, unicode_literals
 import time
 from math import sqrt, pi
 
+from Onboard.Version import require_gi_versions
+require_gi_versions()
 from gi.repository import GLib, Gtk, Gdk
 
-from Onboard.utils import Rect, Timer
-from Onboard.definitions import Handle
+from Onboard.utils import Rect, Version
+from Onboard.Timer import Timer
+from Onboard.definitions import Handle, HandleFunction
 
 import Onboard.osk as osk
 
 ### Logging ###
 import logging
-from functools import reduce
 _logger = logging.getLogger("WindowUtils")
 ###############
 
-from Onboard.Config import Config
-config = Config()
 
 class WindowManipulator(object):
     """
@@ -56,7 +74,7 @@ class WindowManipulator(object):
         self._drag_start_offset  = None
         self._drag_start_rect    = None
         self._drag_handle        = None
-        self._drag_handles = Handle.RESIZERS
+        self._drag_handles = Handle.ALL
         self._drag_active        = False  # has window move/resize actually started yet?
         self._drag_threshold     = 8
         self._drag_snap_threshold = 16
@@ -83,9 +101,12 @@ class WindowManipulator(object):
         self._temporary_unlock_time = None
 
     def get_resize_frame_rect(self):
-        return Rect(0, 0,
-                    self.get_allocated_width(),
-                    self.get_allocated_height())
+        try:
+            return self.get_keyboard_frame_rect()
+        except AttributeError:
+            return Rect(0, 0,
+                        self.get_allocated_width(),
+                        self.get_allocated_height())
 
     def get_drag_start_rect(self):
         return self._drag_start_rect
@@ -99,6 +120,9 @@ class WindowManipulator(object):
     def set_drag_handles(self, handles):
         self._drag_handles = handles
 
+    def get_handle_function(self, handle):
+        return HandleFunction.NORMAL    
+
     def get_drag_threshold(self):
         return 8
 
@@ -121,9 +145,15 @@ class WindowManipulator(object):
                 self.start_move_window(sequence.root_point)
             else:
                 self.start_resize_window(hit, sequence.root_point)
+                
+                function = self.get_handle_function(hit)
+                if function == HandleFunction.ASPECT_RATIO:
+                    self.on_handle_aspect_ratio_pressed()
+                    
             return True
 
-        if move_on_background:
+        if move_on_background and \
+            Handle.MOVE in self.get_drag_handles():
             self.start_move_window(sequence.root_point)
             return True
 
@@ -146,7 +176,7 @@ class WindowManipulator(object):
             if self.drag_protection:
                 # snap off for temporary unlocking
                 if self._temporary_unlock_time is None and \
-                   d > self._drag_threshold:
+                   d >= self._drag_threshold:
                     self._temporary_unlock_time = 1
 
                     # Snap to cursor position for large drag thresholds
@@ -215,46 +245,68 @@ class WindowManipulator(object):
         if not self.is_drag_initiated():
             return
 
-        wx = self._drag_start_pointer[0] + dx - self._drag_start_offset[0]
-        wy = self._drag_start_pointer[1] + dy - self._drag_start_offset[1]
+        function = self.get_handle_function(self._drag_handle)
+        if function == HandleFunction.ASPECT_RATIO:
+        
+            if self._drag_handle == Handle.WEST:
+                dx *= -1
 
-        if self._drag_handle == Handle.MOVE:
-            # contrain axis movement
-            if self._lock_x_axis:
-                wx = self.get_drag_window().get_position()[0]
-            if self._lock_y_axis:
-                wx = self.get_drag_window().get_position()[1]
-
-            # move window
-            x, y = self.limit_position(wx, wy)
-            w, h = None, None
+            self.on_handle_aspect_ratio_motion(dx, dy)
         else:
-            # resize window
-            wmin, hmin = self.get_min_window_size()
-            rect = self._drag_start_rect
-            x0, y0, x1, y1 = rect.to_extents()
-            w, h = rect.get_size()
+            wx = self._drag_start_pointer[0] + dx - self._drag_start_offset[0]
+            wy = self._drag_start_pointer[1] + dy - self._drag_start_offset[1]
 
-            if self._drag_handle in [Handle.NORTH,
-                                     Handle.NORTH_WEST,
-                                     Handle.NORTH_EAST]:
-                y0 = min(wy, y1 - hmin)
-            if self._drag_handle in [Handle.WEST,
-                                     Handle.NORTH_WEST,
-                                     Handle.SOUTH_WEST]:
-                x0 = min(wx, x1 - wmin)
-            if self._drag_handle in [Handle.EAST,
-                                     Handle.NORTH_EAST,
-                                     Handle.SOUTH_EAST]:
-                x1 = max(wx + w, x0 + wmin)
-            if self._drag_handle in [Handle.SOUTH,
-                                     Handle.SOUTH_WEST,
-                                     Handle.SOUTH_EAST]:
-                y1 = max(wy + h, y0 + hmin)
+            if self._drag_handle == Handle.MOVE:
+                # contrain axis movement
+                if self._lock_x_axis:
+                    wx = self.get_drag_window().get_position()[0]
+                if self._lock_y_axis:
+                    wx = self.get_drag_window().get_position()[1]
 
-            x, y, w, h = x0, y0, x1 -x0, y1 - y0
+                # move window
+                x, y = self.limit_position(wx, wy)
+                w, h = None, None
+            else:
+                # resize window
+                wmin, hmin = self.get_min_window_size()
+                rect = self._drag_start_rect
+                x0, y0, x1, y1 = rect.to_extents()
+                w, h = rect.get_size()
 
-        self._move_resize(x, y, w, h)
+                if self._drag_handle in [Handle.NORTH,
+                                        Handle.NORTH_WEST,
+                                        Handle.NORTH_EAST]:
+                    y0 = min(wy, y1 - hmin)
+                if self._drag_handle in [Handle.WEST,
+                                        Handle.NORTH_WEST,
+                                        Handle.SOUTH_WEST]:
+                    x0 = min(wx, x1 - wmin)
+                if self._drag_handle in [Handle.EAST,
+                                        Handle.NORTH_EAST,
+                                        Handle.SOUTH_EAST]:
+                    x1 = max(wx + w, x0 + wmin)
+                if self._drag_handle in [Handle.SOUTH,
+                                        Handle.SOUTH_WEST,
+                                        Handle.SOUTH_EAST]:
+                    y1 = max(wy + h, y0 + hmin)
+
+                x, y, w, h = x0, y0, x1 -x0, y1 - y0
+
+            self._move_resize(x, y, w, h)
+
+    def on_handle_aspect_ratio_pressed(self):
+        """
+        Overload this to process start of dragging of
+        handles with ASPECT_RATIO function.
+        """
+        pass
+
+    def on_handle_aspect_ratio_motion(self, wx, wy):
+        """
+        Overload this to process motion changes of 
+        handles with ASPECT_RATIO function.
+        """
+        pass
 
     def set_drag_cursor_at(self, point, allow_drag_cursors = True):
         """ set the mouse cursor """
@@ -281,7 +333,7 @@ class WindowManipulator(object):
         if not window:
             return
 
-        if not self._drag_handle is None:    # not dragging a handle?
+        if self._drag_handle is None:    # not dragging a handle?
             window.set_cursor(None)
 
     def get_drag_cursor_at(self, point):
@@ -424,15 +476,16 @@ class WindowManipulator(object):
 
     def limit_size(self, rect):
         """
-        Limits the given window rect to fit on screen.
+        Limit the given window rect to fit on screen.
         """
         screen = self.get_screen()
         limits = Rect(0, 0, screen.get_width(), screen.get_height())
         r = rect.copy()
-        if r.w > limits.w:
-            r.w = limits.w - 40
-        if r.h > limits.h:
-            r.h = limits.h - 20
+        if not limits.is_empty():  # LP #1633284
+            if r.w > limits.w:
+                r.w = limits.w - 40
+            if r.h > limits.h:
+                r.h = limits.h - 20
         return r
 
     def limit_position(self, x, y, visible_rect = None, limit_rects = None):
@@ -488,10 +541,10 @@ class WindowManipulator(object):
         # then check the edges
         for handle in handles:
             if handle == Handle.WEST:
-                if x < x0 + w:
+                if x < x0 + w and x >= x0 - 1:
                     return handle
             if handle == Handle.EAST:
-                if x > x1 - w:
+                if x > x1 - w and x <= x1 + 1:
                     return handle
             if handle == Handle.NORTH:
                 if y < y0 + h:
@@ -499,7 +552,7 @@ class WindowManipulator(object):
             if handle == Handle.SOUTH:
                 if y > y1 - h:
                     return handle
-
+                    
         return None
 
     def _move_resize(self, x, y, w = None, h = None):
@@ -588,6 +641,9 @@ class WindowRectTracker:
             if len(origin) == 3:   # What is the first parameter for? Gdk bug?
                 origin = origin[1:]
 
+            if _logger.isEnabledFor(logging.DEBUG):
+                _logger.debug("update_window_rect1: pos {}, size {}, origin {}"
+                         .format(pos, size, origin))
             pos = self._apply_window_scaling_factor(pos)
 
             self._window_rect = Rect.from_position_size(pos, size)
@@ -595,6 +651,13 @@ class WindowRectTracker:
             self._client_offset = (origin[0] - pos[0], origin[1] - pos[1])
             self._screen_orientation = self.get_screen_orientation()
 
+            if _logger.isEnabledFor(logging.DEBUG):
+                _logger.debug("update_window_rect2: pos {}, client_offset {}, "
+                              "screen_orientation {}"
+                              .format(pos,
+                                      self._client_offset,
+                                      self._screen_orientation))
+
     def move(self, x, y):
         Gtk.Window.move(self, x, y)
 
@@ -665,6 +728,19 @@ class WindowRectTracker:
                 values = (values[0] * scale, values[1] * scale)
         return values
 
+    def get_screen_orientation(self):
+        """
+        Current orientation of the screen (tablet rotation).
+        Only the aspect ratio is taken into account at this time.
+        This appears to cover more cases than looking at monitor rotation,
+        in particular with multi-monitor screens.
+        """
+        screen = self.get_screen()
+        if screen.get_width() >= screen.get_height():
+            return Orientation.LANDSCAPE
+        else:
+            return Orientation.PORTRAIT
+
 
 class WindowRectPersist(WindowRectTracker):
     """
@@ -698,19 +774,6 @@ class WindowRectPersist(WindowRectTracker):
     def on_screen_size_changed_delayed(self, screen):
         self.restore_window_rect()
 
-    def get_screen_orientation(self):
-        """
-        Current orientation of the screen (tablet rotation).
-        Only the aspect ratio is taken into account at this time.
-        This appears to cover more cases than looking at monitor rotation,
-        in particular with multi-monitor screens.
-        """
-        screen = self.get_screen()
-        if screen.get_width() >= screen.get_height():
-            return Orientation.LANDSCAPE
-        else:
-            return Orientation.PORTRAIT
-
     def restore_window_rect(self, startup = False):
         """
         Restore window size and position.
@@ -754,7 +817,7 @@ class WindowRectPersist(WindowRectTracker):
     def on_restore_window_rect(self, rect):
         return rect
 
-    def save_window_rect(self, orientation = None, rect = None):
+    def save_window_rect(self, orientation=None, rect=None):
         """
         Save window size and position.
         """
@@ -770,7 +833,7 @@ class WindowRectPersist(WindowRectTracker):
         self.write_window_rect(orientation, rect)
 
         _logger.debug("save_window_rect {rect}, {orientation}" \
-                      .format(rect = rect, orientation = orientation))
+                      .format(rect=rect, orientation=orientation))
 
     def on_save_window_rect(self, rect):
         return rect
@@ -797,9 +860,10 @@ class WindowRectPersist(WindowRectTracker):
         Remember the current rect and rotation as the screen may have been
         rotated when the saving happens.
         """
-        self._save_position_timer.start(5, self.save_window_rect,
-                                           self.get_screen_orientation(),
-                                           self.get_rect())
+        self._save_position_timer.start(5,
+                                        self.save_window_rect,
+                                        self.get_screen_orientation(),
+                                        self.get_rect())
 
     def stop_save_position_timer(self):
         self._save_position_timer.stop()
@@ -814,7 +878,7 @@ def set_unity_property(window):
     """
     gdk_win = window.get_window()
     if gdk_win:
-        if hasattr(gdk_win, "get_xid"): # not on wayland
+        if hasattr(gdk_win, "get_xid"):  # not on wayland
             xid = gdk_win.get_xid()
             osk.Util().set_x_property(xid, "ONSCREEN_KEYBOARD", 1)
 
@@ -830,7 +894,7 @@ class DwellProgress(object):
     opacity = 1.0
 
     def is_dwelling(self):
-        return not self.dwell_start_time is None
+        return self.dwell_start_time is not None
 
     def is_done(self):
         return time.time() > self.dwell_start_time + self.dwell_delay
@@ -841,7 +905,7 @@ class DwellProgress(object):
     def stop_dwelling(self):
         self.dwell_start_time = None
 
-    def draw(self, context, rect, rgba = (1, 0, 0, .75), rgba_bg = None):
+    def draw(self, context, rect, rgba=(1, 0, 0, .75), rgba_bg = None):
         if self.is_dwelling():
             if self.opacity <= 0.0:
                 pass
@@ -856,7 +920,7 @@ class DwellProgress(object):
                 self._draw_dwell_progress(context, rect, rgba, rgba_bg)
 
                 context.pop_group_to_source()
-                context.paint_with_alpha(self.opacity);
+                context.paint_with_alpha(self.opacity)
                 context.restore()
 
     def _draw_dwell_progress(self, context, rect, rgba, rgba_bg):
@@ -883,7 +947,7 @@ class DwellProgress(object):
             context.set_source_rgba(*rgba)
             context.fill_preserve()
 
-            context.set_source_rgba(0,0,0,1)
+            context.set_source_rgba(0, 0, 0, 1)
             context.set_line_width(0)
             context.stroke()
 
@@ -896,8 +960,8 @@ def limit_window_position(x, y, always_visible_rect, limit_rects = None):
     # rect to stay always visible, in canvas coordinates
     r = always_visible_rect
 
-    if not r is None:
-        r = r.int() # avoid rounding errors
+    if r is not None:
+        r = r.int()  # avoid rounding errors
 
         # transform always visible rect to screen coordinates,
         # take window decoration into account.
@@ -968,7 +1032,7 @@ def canvas_to_root_window_point(window, point):
     if gdk_win:
         point = gdk_win.get_root_coords(*point)
     else:
-        point (0, 0)
+        point(0, 0)
     return point
 
 def get_monitor_dimensions(window):
@@ -984,7 +1048,7 @@ def get_monitor_dimensions(window):
 
         # Nexus7 simulation
         device = None       # keep this at None
-        #device = 1
+        # device = 1
         if device == 0:     # dimension unavailable
             size_mm = 0, 0
         elif device == 1:     # Nexus 7, as it should report
@@ -993,19 +1057,105 @@ def get_monitor_dimensions(window):
 
         return size, size_mm
     else:
-        return None, None
+        return (0, 0), (0, 0)
 
 def physical_to_monitor_pixel_size(window, size_mm, fallback_size = (0, 0)):
     """
     Convert a physical size in mm to pixels of windows's monitor,
     """
     sz, sz_mm = get_monitor_dimensions(window)
-    if sz and sz_mm:
+    if sz[0] > 0 and sz[1] > 0 and \
+       sz_mm[0] > 0 and sz_mm[1] > 0:
         w = sz[0] * size_mm[0] / sz_mm[0] \
             if sz_mm[0] else fallback_size[0]
         h = sz[1] * size_mm[1] / sz_mm[1] \
             if sz_mm[0] else fallback_size[1]
     else:
-        w = h = 0
+        w, h = fallback_size
     return w, h
 
+def show_error_dialog(error_string):
+    """ Show an error dialog """
+
+    error_dlg = Gtk.MessageDialog(message_type=Gtk.MessageType.ERROR,
+                                  message_format=error_string,
+                                  buttons=Gtk.ButtonsType.OK)
+    error_dlg.run()
+    error_dlg.destroy()
+
+def show_ask_string_dialog(question, parent=None):
+    question_dialog = Gtk.MessageDialog(message_type=Gtk.MessageType.QUESTION,
+                                        buttons=Gtk.ButtonsType.OK_CANCEL)
+    if parent:
+        question_dialog.set_transient_for(parent)
+    question_dialog.set_markup(question)
+    entry = Gtk.Entry()
+    entry.connect("activate", lambda event:
+        question_dialog.response(Gtk.ResponseType.OK))
+    question_dialog.get_message_area().add(entry)
+    question_dialog.show_all()
+    response = question_dialog.run()
+    text = entry.get_text() if response == Gtk.ResponseType.OK else None
+    question_dialog.destroy()
+    return text
+
+def show_confirmation_dialog(question, parent=None, center=False, title=None):
+    """
+    Show this dialog to ask confirmation before executing a task.
+    """
+    if title is None:
+        # Default dialog title: name of the application
+        title = _("Onboard")
+    dlg = Gtk.MessageDialog(message_type=Gtk.MessageType.QUESTION,
+                            text=question,
+                            title=title,
+                            buttons=Gtk.ButtonsType.YES_NO)
+    if parent:
+        dlg.set_transient_for(parent)
+
+    if center:
+        dlg.set_position(Gtk.WindowPosition.CENTER)
+
+    response = dlg.run()
+    dlg.destroy()
+    return response == Gtk.ResponseType.YES
+
+def show_new_device_dialog(name, config_string, is_pointer, callback):
+    """
+    Show a "New Input Device" dialog.
+    """
+    dialog = Gtk.MessageDialog(message_type=Gtk.MessageType.OTHER,
+                               title=_("New Input Device"),
+                               text=_("Onboard has detected a new input device"))
+    if is_pointer:
+        dialog.set_image(Gtk.Image(icon_name="input-mouse",
+                                   icon_size=Gtk.IconSize.DIALOG))
+    else:
+        dialog.set_image(Gtk.Image(icon_name="input-keyboard",
+                                   icon_size=Gtk.IconSize.DIALOG))
+
+    secondary  = "<i>{}</i>\n\n".format(name)
+    secondary += _("Do you want to use this device for keyboard scanning?")
+
+    dialog.format_secondary_markup(secondary)
+
+    # Translators: cancel button of "New Input Device" dialog. It used to be
+    # stock item STOCK_CANCEL until Gtk 3.10 deprecated those.
+    dialog.add_button(_("_Cancel"), Gtk.ResponseType.CANCEL)
+    dialog.add_button(_("Use device"), Gtk.ResponseType.ACCEPT).grab_default()
+    dialog.connect("response", _show_new_device_dialog_response,
+                   callback, config_string)
+    dialog.show_all()
+
+def _show_new_device_dialog_response(dialog, response, callback, config_string):
+    """ Callback for the "New Input Device" dialog. """
+    if response == Gtk.ResponseType.ACCEPT:
+        callback(config_string)
+    dialog.destroy()
+
+def gtk_has_resize_grip_support():
+    """ Gtk from 3.14 removes resize grips. """
+    gtk_version = Version(Gtk.MAJOR_VERSION, Gtk.MINOR_VERSION)
+    return gtk_version < Version(3, 14)
+
+
