@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright © 2012, marmuta
+# Copyright © 2012-2017 marmuta <marmvta@gmail.com>
 #
 # This file is part of Onboard.
 #
@@ -22,9 +22,12 @@ from __future__ import division, print_function, unicode_literals
 import sys
 import copy
 
-from gi.repository import Gdk
+from Onboard.Version        import require_gi_versions
+require_gi_versions()
+from gi.repository          import Gdk
 
-from Onboard.utils import EventSource, unicode_str
+from Onboard.utils          import EventSource, unicode_str
+from Onboard.definitions    import UINPUT_DEVICE_NAME
 
 import Onboard.osk as osk
 
@@ -122,7 +125,9 @@ class XIDeviceManager(EventSource):
     XInput device manager singleton.
     """
 
-    blacklist = ("Virtual core XTEST keyboard", "Power Button")
+    blacklist = ("Virtual core XTEST keyboard",
+                 UINPUT_DEVICE_NAME,
+                 "Power Button")
     last_device_blacklist = ("Virtual core XTEST pointer")
 
     def __new__(cls, *args, **kwargs):
@@ -145,7 +150,8 @@ class XIDeviceManager(EventSource):
         """
         Singleton constructor, runs only once.
         """
-        EventSource.__init__(self, ["device-event", "device-grab"])
+        EventSource.__init__(self, ["device-event", "device-grab",
+                                    "devices-updated"])
 
         self._devices = {}
         self._osk_devices = None
@@ -182,7 +188,7 @@ class XIDeviceManager(EventSource):
         return self.lookup_device_id(device_id)
 
     def get_client_keyboard(self):
-        """ Return client pointer device """
+        """ Return client keyboard device """
         client_pointer = self.get_client_pointer()
         device_id = client_pointer.attachment
         return self.lookup_device_id(device_id)
@@ -274,14 +280,16 @@ class XIDeviceManager(EventSource):
         self._last_click_device_id = None
         self._last_motion_device_id = None
 
+        # notify listeners about the previous devices becoming invalid
+        self.emit("devices-updated")
+
     def select_events(self, window, device, mask):
         if window is None:  # use root window?
             xid = 0
         else:
-            win = window.get_window()
-            if not win:
-                return False # no gdk window yet
-            xid = win.get_xid()
+            xid = window.get_xid()
+            if not xid:
+                return False
 
         self._osk_devices.select_events(xid, device.id, mask)
         return True
@@ -290,10 +298,9 @@ class XIDeviceManager(EventSource):
         if window is None:  # use root window?
             xid = 0
         else:
-            win = window.get_window()
-            if not win:
-                return False # no gdk window yet
-            xid = win.get_xid()
+            xid = window.get_xid()
+            if not xid:
+                return False
 
         self._osk_devices.unselect_events(xid, device.id)
         return True
@@ -438,11 +445,6 @@ class XIDevice(object):
     source       = None
     touch_mode   = None
 
-    touch_active = False  # If False, only pointer events are considered.
-                          # Wacom devices with enabled gestures never
-                          # become touch-active, i.e. they don't generate
-                          # touch events.
-
     _device_manager = None
 
     def __repr__(self):
@@ -557,11 +559,7 @@ class XIDeviceEventLogger:
         if not _logger.isEnabledFor(logging.DEBUG):
             self.log_event = self._log_event_stub
 
-    def get_window(self):
-        return None
-
     def _log_device_event(self, event):
-        win = self.get_window()
         if not event.xi_type in [ XIEventType.TouchUpdate,
                                   XIEventType.Motion]:
             self.log_event("Device event: dev_id={} src_id={} xi_type={} "
@@ -571,7 +569,7 @@ class XIDeviceEventLogger:
                                     event.source_id,
                                     event.xi_type,
                                     event.xid_event,
-                                    win.get_xid() if win else 0,
+                                    self.get_xid(),
                                     event.x, event.y,
                                     event.x_root, event.y_root,
                                     event.button, event.state,
