@@ -1,25 +1,43 @@
 # -*- coding: utf-8 -*-
 
+# Copyright © 2007-2009 Chris Jones <tortoise@tortuga>
+# Copyright © 2008, 2010 Francesco Fumanti <francesco.fumanti@gmx.net>
+# Copyright © 2012 Gerd Kohlberger <lowfi@chello.at>
+# Copyright © 2011-2016 marmuta <marmvta@gmail.com>
+#
+# This file is part of Onboard.
+#
+# Onboard is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 3 of the License, or
+# (at your option) any later version.
+#
+# Onboard is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program. If not, see <http://www.gnu.org/licenses/>.
+
 from __future__ import division, print_function, unicode_literals
 
 import sys
 import os
 import time
 import re
-import traceback
 import colorsys
 import gettext
-from subprocess import Popen
+import subprocess
 from math import pi, sin, cos, sqrt, log
 from contextlib import contextmanager
 
-from gi.repository import GLib, Gtk
-
-### Logging ###
 import logging
 from functools import reduce
+
+from gi.repository import GLib
+
 _logger = logging.getLogger("utils")
-###############
 
 
 # keycodes
@@ -28,6 +46,7 @@ class KeyCode:
     KP_Enter = 104
     C        = 54
 
+
 class Modifiers:
     # 1      2     4    8    16     32    64     128
     SHIFT, CAPS, CTRL, ALT, NUMLK, MOD3, SUPER, ALTGR = \
@@ -110,7 +129,8 @@ funcKeys = (("ESC",65307),
             ("Scroll", 65300),
             ("Pause", 65299))
 
-keysyms = {"space" : 65408,
+keysyms = {"space" : 0x0020,
+           "kp_space" : 65408,
            "insert" : 0xff63,
            "home" : 0xff50,
            "page_up" : 0xff55,
@@ -332,78 +352,6 @@ class dictproperty(object):
             return self
         return self._proxy(obj, self._fget, self._fset, self._fdel)
 
-def show_error_dialog(error_string):
-    """ Show an error dialog """
-
-    error_dlg = Gtk.MessageDialog(message_type=Gtk.MessageType.ERROR,
-                                  message_format=error_string,
-                                  buttons=Gtk.ButtonsType.OK)
-    error_dlg.run()
-    error_dlg.destroy()
-
-def show_ask_string_dialog(question, parent=None):
-    question_dialog = Gtk.MessageDialog(message_type=Gtk.MessageType.QUESTION,
-                                        buttons=Gtk.ButtonsType.OK_CANCEL)
-    if parent:
-        question_dialog.set_transient_for(parent)
-    question_dialog.set_markup(question)
-    entry = Gtk.Entry()
-    entry.connect("activate", lambda event:
-        question_dialog.response(Gtk.ResponseType.OK))
-    question_dialog.get_message_area().add(entry)
-    question_dialog.show_all()
-    response = question_dialog.run()
-    text = entry.get_text() if response == Gtk.ResponseType.OK else None
-    question_dialog.destroy()
-    return text
-
-def show_confirmation_dialog(question, parent=None, center=False):
-    """
-    Show this dialog to ask confirmation before executing a task.
-    """
-    dlg = Gtk.MessageDialog(message_type=Gtk.MessageType.QUESTION,
-                            text=question,
-                            buttons=Gtk.ButtonsType.YES_NO)
-    if parent:
-        dlg.set_transient_for(parent)
-
-    if center:
-        dlg.set_position(Gtk.WindowPosition.CENTER)
-
-    response = dlg.run()
-    dlg.destroy()
-    return response == Gtk.ResponseType.YES
-
-def show_new_device_dialog(name, config_string, is_pointer, callback):
-    """
-    Show a "New Input Device" dialog.
-    """
-    dialog = Gtk.MessageDialog(message_type  = Gtk.MessageType.OTHER,
-                               title = _("New Input Device"),
-                               text  = _("Onboard has detected a new input device"))
-    if is_pointer:
-        dialog.set_image(Gtk.Image(icon_name = "input-mouse",
-                                   icon_size = Gtk.IconSize.DIALOG))
-    else:
-        dialog.set_image(Gtk.Image(icon_name = "input-keyboard",
-                                   icon_size = Gtk.IconSize.DIALOG))
-
-    secondary  = "<i>{}</i>\n\n".format(name)
-    secondary += _("Do you want to use this device for keyboard scanning?")
-
-    dialog.format_secondary_markup(secondary)
-    dialog.add_button(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL)
-    dialog.add_button(_("Use device"), Gtk.ResponseType.ACCEPT).grab_default()
-    dialog.connect("response", _show_new_device_dialog_response,
-                   callback, config_string)
-    dialog.show_all()
-
-def _show_new_device_dialog_response(dialog, response, callback, config_string):
-    """ Callback for the "New Input Device" dialog. """
-    if response == Gtk.ResponseType.ACCEPT:
-        callback(config_string)
-    dialog.destroy()
-
 def unpack_name_value_list(_list, num_values=2, key_type = str):
     """
     Converts a list of strings into a dict of tuples.
@@ -474,53 +422,6 @@ def merge_tuple_strings(text1, text2):
     return pack_name_value_tuples(tuples1)
 
 
-class CallOnce(object):
-    """
-    Call each <callback> during <delay> only once
-    Useful to reduce a storm of config notifications
-    to just a single (or a few) update(s) of onboards state.
-    """
-
-    def __init__(self, delay=20, delay_forever=False):
-        self.callbacks = {}
-        self.timer = None
-        self.delay = delay
-        self.delay_forever = delay_forever
-
-    def is_running(self):
-        return not self.timer is None
-
-    def enqueue(self, callback, *args):
-        if not callback in self.callbacks:
-            self.callbacks[callback] = args
-        else:
-            #print "CallOnce: ignored ", callback, args
-            pass
-
-        if self.delay_forever and self.timer:
-            GLib.source_remove(self.timer)
-            self.timer = None
-
-        if not self.timer and self.callbacks:
-            self.timer = GLib.timeout_add(self.delay, self.cb_timer)
-
-    def stop(self):
-        if self.timer:
-            GLib.source_remove(self.timer)
-            self.timer = None
-        self.callbacks.clear()
-
-    def cb_timer(self):
-        for callback, args in list(self.callbacks.items()):
-            try:
-                callback(*args)
-            except:
-                traceback.print_exc()
-
-        self.stop()
-        return False
-
-
 class Rect:
     """
     Simple rectangle class.
@@ -553,6 +454,9 @@ class Rect:
                      for a in self.attributes) + \
             ")"
 
+    def __repr__(self):
+        return self.__str__()
+
     def __eq__(self, other):
         return self.x == other.x and \
                self.y == other.y and \
@@ -744,7 +648,7 @@ class Rect:
 
     def resize_to_aspect(self, aspect_rect):
         """
-        Resize self to get the aspect ratio of aspect_rect.
+        Resize self to the aspect ratio of aspect_rect.
         """
         if self.is_empty() or aspect_rect.is_empty():
             return Rect()
@@ -762,7 +666,7 @@ class Rect:
     def resize_to_aspect_range(self, aspect_rect, aspect_change_range):
         """
         Resize self to get the aspect ratio of aspect_rect, but limited
-        but the given aspect range.
+        to the given aspect range.
         """
         if self.is_empty() or aspect_rect.is_empty():
             return Rect()
@@ -1069,150 +973,6 @@ def timeit(s, out=sys.stdout):
     else:
         yield None
 
-
-class Timer(object):
-    """
-    Simple wrapper around GLib's timer API
-    Overload on_timer in derived classes.
-    For one-shot timers return False there.
-    """
-    _timer = None
-    _callback = None
-    _callback_args = None
-
-    def __init__(self, delay = None, callback = None, *callback_args):
-        self._callback = callback
-        self._callback_args = callback_args
-
-        if not delay is None:
-            self.start(delay)
-
-    def start(self, delay, callback = None, *callback_args):
-        """
-        Delay in seconds.
-        Uses second granularity if delay is of type int.
-        Uses medium resolution timer if delay is of type float.
-        """
-        if callback:
-            self._callback = callback
-            self._callback_args = callback_args
-
-        self.stop()
-
-        if type(delay) == int:
-            self._timer = GLib.timeout_add_seconds(delay, self._cb_timer)
-        else:
-            ms = int(delay * 1000.0)
-            self._timer = GLib.timeout_add(ms, self._cb_timer)
-
-    def finish(self):
-        """
-        Run one last time and stop.
-        """
-        if self.is_running():
-            self.stop()
-            self.on_timer()
-
-    def stop(self):
-        if self.is_running():
-            GLib.source_remove(self._timer)
-            self._timer = None
-
-    def is_running(self):
-        return self._timer is not None
-
-    def _cb_timer(self):
-        if not self.on_timer():
-            self.stop()
-            return False
-        return True
-
-    def on_timer(self):
-        """
-        Overload this.
-        For one-shot timers return False.
-        """
-        if self._callback:
-            return self._callback(*self._callback_args)
-        return True
-
-
-class TimerOnce(Timer):
-    def on_timer(self):
-        """
-        Overload this.
-        """
-        if self._callback:
-            return self._callback(*self._callback_args)
-        return False
-
-
-class DelayedLauncher(Timer):
-    """
-    Launches a process after a certain delay.
-    Used for launching mousetweaks.
-    """
-    args = None
-
-    def launch_delayed(self, args, delay):
-        self.args = args
-        self.start(delay)
-
-    def on_timer(self):
-        _logger.debug("launching '{}'".format(" ".join(self.args)))
-        try:
-            Popen(self.args)
-        except OSError as e:
-            _logger.warning(_format("Failed to execute '{}', {}", \
-                            " ".join(self.args), e))
-        return False
-
-
-class FadeTimer(Timer):
-    """
-    Sine-interpolated fade between two values, e.g. opacities.
-    """
-    value = None
-    start_value = None
-    target_value = None
-    iteration = 0   # just a counter of on_timer calls since start
-    time_step = 0.05
-
-    def fade_to(self, start_value, target_value, duration,
-                callback = None, *callback_args):
-        """
-        Start value fade.
-        duration: fade time in seconds, 0 for immediate value change
-        """
-        self.value = start_value
-        self.start_value = start_value
-        self._start_time = time.time()
-        self._duration = duration
-        self._callback = callback
-        self._callback_args = callback_args
-
-        self.start(self.time_step)
-
-        self.target_value = target_value
-
-    def start(self, delay):
-        self.iteration = 0
-        Timer.start(self, delay)
-
-    def stop(self):
-        self.target_value = None
-        Timer.stop(self)
-
-    def on_timer(self):
-        self.value, done = Fade.sin_fade(self._start_time, self._duration,
-                                         self.start_value, self.target_value)
-
-        if self._callback:
-            self._callback(self.value, done, *self._callback_args)
-
-        self.iteration += 1
-        return not done
-
 class Fade:
     """ Helper for opacity fading """
     @staticmethod
@@ -1369,23 +1129,30 @@ class Process:
         cmdline = ""
         with open("/proc/%s/cmdline" % pid) as f:
             cmdline = f.read()
-        return cmdline
+        return cmdline.split("\0")
 
     @staticmethod
     def get_process_name(pid):
-        args = Process.get_cmdline(pid).split("\0")
-        if args:
-            return os.path.basename(args[0])
+        cmdline = Process.get_cmdline(pid)
+        if cmdline:
+            return os.path.basename(cmdline[0])
         return ""
 
     @staticmethod
-    def was_launched_by(process_name):
+    def get_launch_process_cmdline():
         """ Checks if this process was launched by <process_name> """
         ppid = os.getppid()
         if ppid:
             cmdline = Process.get_cmdline(ppid)
-            return process_name in cmdline
-        return False
+            return cmdline
+        return []
+
+    @staticmethod
+    def was_launched_by(process_name):
+        """ Checks if this process was launched by <process_name> """
+        cmdline = " ".join(Process.get_launch_process_cmdline())
+        return process_name in cmdline
+
 
 def exists_in_path(basename):
     """
@@ -1488,6 +1255,8 @@ class Translation:
         builtins.__dict__['_format'] = t.format
 
     def ugettext(self, msgid):
+        if not msgid:
+            return unicode_str("")
         if sys.version_info.major < 3:  # python 2?
             return self.translation.ugettext(msgid)
         return self.translation.gettext(msgid)
@@ -1515,7 +1284,7 @@ class EventSource(object):
     def __init__(self, event_names):
         self._callbacks = dict((e,[]) for e in event_names)
 
-    def cleanup():
+    def cleanup(self):
         self.flush_events()
 
     def connect(self, event_name, callback):
@@ -1579,6 +1348,7 @@ class XDGDirs:
 
     Doctests:
 
+    >>> old_env = os.environ.copy()
     >>> os.environ["HOME"] = "/home/test_user"
 
     # XDG_CONFIG_HOME unavailable
@@ -1631,6 +1401,8 @@ class XDGDirs:
 '/usr/share/gnome/onboard/test.dat', \
 '/usr/local/share/onboard/test.dat', \
 '/usr/share/onboard/test.dat']
+
+    >>> os.environ = old_env
     """
 
     @staticmethod
@@ -1766,3 +1538,143 @@ class XDGDirs:
         return exists
 
 
+_tag_pattern = re.compile(
+    """(?:
+            <[\w\-_]+                         # tag
+            (?:\s+[\w\-_]+=["'][^"']*["'])*  # attributes
+            /?>
+        ) |
+        (?:
+            </?[\w\-_]+>
+        )
+    """, re.UNICODE|re.DOTALL|re.VERBOSE)
+
+def _iter_markup(markup):
+    """
+    Iterate over tag and non-tag sections of a markup string.
+
+    Doctests:
+    # Never yield for empty string
+    >>> list(_iter_markup(""))
+    []
+
+    # must return tag- as well as non-tag sections
+    >>> list(_iter_markup("<tt>test</tt>test2"))
+    [('<tt>', True), ('test', False), ('</tt>', True), ('test2', False)]
+
+    # should recognize tags with attributes
+    >>> list(_iter_markup('<tag attr="value">'))
+    [('<tag attr="value">', True)]
+    >>> list(_iter_markup('<tag attr="v alue" attr2="234">'))
+    [('<tag attr="v alue" attr2="234">', True)]
+
+    # should recognize tags with end shortcut
+    >>> list(_iter_markup('<tag/> <tag2 attr="value"/>'))
+    [('<tag/>', True), (' ', False), ('<tag2 attr="value"/>', True)]
+
+    # must not modify input, i.e. concatenated result must equal input text
+    >>> markup = "asd <tt>t est\\n ds</tt> te st2 "
+    >>> "".join([text for text, tag in _iter_markup(markup)]) == markup
+    True
+    """
+    pos = 0
+    matches = _tag_pattern.finditer(markup)
+    for m in matches:
+        text = markup[pos:m.start()]
+        if text:
+            yield text, False
+        yield m.group(), True
+        pos = m.end()
+
+    text = markup[pos:]
+    if text:
+        yield text, False
+
+
+def escape_markup(markup, preserve_tags = False):
+    """
+    Escape strings of uncertain content for use in Gtk markup.
+    If requested, markup tags are skipped and won't be escaped.
+
+    Doctests:
+    >>> escape_markup("&<>")
+    '&amp;&lt;&gt;'
+
+    # tags must be escaped when preserve_tags is False
+    >>> escape_markup("<big>&<></big>")
+    '&lt;big&gt;&amp;&lt;&gt;&lt;/big&gt;'
+
+    # tags must not be escaped when preserve_tags is True
+    >>> escape_markup('<big>&</big>><tag attr="value"><</tag>1<3', True)
+    '<big>&amp;</big>&gt;<tag attr="value">&lt;</tag>1&lt;3'
+
+    # whitespace must be preserved
+    >>> escape_markup("test <big> test2& </big> test3", True)
+    'test <big> test2&amp; </big> test3'
+    """
+    result = ""
+    for text, is_tag in _iter_markup(markup):
+        if is_tag and preserve_tags:
+            result += text
+        else:
+            try:
+                result += GLib.markup_escape_text(text)
+            except Exception as ex: # private exception gi._glib.GError
+                _logger.error("markup_escape_text failed for "
+                                "'{}': {}" \
+                            .format(text, unicode_str(ex)))
+    return result
+
+
+class TermColors(object):
+    """ Singleton class providing ANSI terminal color codes """
+
+    # sequence ids
+    BLACK, RED, GREEN, YELLOW, BLUE, MAGENTA, CYAN, WHITE, \
+    BOLD, RESET = range(10)
+
+    # sequence cache
+    sequences = {}
+
+    def __new__(cls, *args, **kwargs):
+        """ Singleton magic.  """
+        if not hasattr(cls, "self"):
+            cls.self = object.__new__(cls, *args, **kwargs)
+            cls.self.construct()
+        return cls.self
+
+    def __init__(self):
+        """ Called multiple times, do not use.  """
+        pass
+
+    def construct(self):
+        """ Singleton constructor, runs only once.  """
+
+    def get(self, seq_id):
+        """
+        Return ANSI character sequence for the given sequence id,
+        e.g. color index.
+        """
+        seq = self.sequences.get(seq_id)
+        if seq is None:
+            seq = ""
+            if not seq_id is None:
+                if seq_id >= self.BLACK and seq_id <= self.WHITE:
+                    seq = self._tput("setaf " + str(seq_id))
+                elif seq_id == self.BOLD:
+                    seq = self._tput("bold")
+                elif seq_id == self.RESET:
+                    seq = self._tput("sgr0")
+            self.sequences[seq_id] = seq
+        return seq
+
+    @staticmethod
+    def _tput(params):
+        try:
+            s = subprocess.check_output(("tput " + params).split())
+            return s.decode("ASCII")
+        except subprocess.CalledProcessError:
+            return ""
+
+
+
